#!/usr/local/bin/ksh93 -p
#
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
# or http://www.opensolaris.org/os/licensing.
# See the License for the specific language governing permissions
# and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each
# file and include the License file at usr/src/OPENSOLARIS.LICENSE.
# If applicable, add the following below this CDDL HEADER, with the
# fields enclosed by brackets "[]" replaced with your own identifying
# information: Portions Copyright [yyyy] [name of copyright owner]
#
# CDDL HEADER END
#

#
# Copyright 2012 Spectra Logic.  All rights reserved.
# Use is subject to license terms.
#
# ident	"@(#)hotspare_replace_005_pos.ksh	1.0	12/08/09 SL"
#
. $STF_SUITE/tests/functional/hotspare/hotspare.kshlib

################################################################################
#
# __stc_assertion_start
#
# ID: hotspare_replace_005_pos
#
# DESCRIPTION: 
#	If a vdev becomes faulted in a pool with a spare, the spare will be
#	activated.
#       
#
# STRATEGY:
#	1. Create 1 storage pools with hot spares.  Use disks with the da
#  	   driver only, because it can inject errors.
#	2. Fail one vdev in one pool to make 1 hotspare in use.
#	3. Verify that the spare is in use.
#
# TESTABILITY: explicit
#
# TEST_AUTOMATION_LEVEL: automated
#
# CODING STATUS: COMPLETED (2012-08-06)
#
# __stc_assertion_end
#
###############################################################################

verify_runnable "global"
verify_runtime $RT_LONG
verify_disk_count "$DISKS" 5

function cleanup
{
	poolexists $TESTPOOL && \
		destroy_pool $TESTPOOL

	partition_cleanup
}

function verify_assertion # spare_dev
{
	typeset sdev=$1

	# Cause some IO errors writing to the pool
	while true
	do
		# Running zpool status after every dd operation is too slow.
		# So we will run several dd's in a row before checking zpool
		# status.  sync between dd operations to ensure that the disk
		# gets IO
		for ((i=0; $i<64; i=$i+1))
		do
			sysctl kern.cam.da.$DADISKNUM.error_inject=1 > /dev/null
			dd if=/dev/zero bs=128k count=1 >> \
				/$TESTPOOL/testfile 2> /dev/null
			sync
		done
		# Check to see if the disk is removed yet
		$ZPOOL status $TESTPOOL | grep -q 'state: DEGRADED'
		if [ $? == 0 ]
		then
			log_note "$TESTPOOL got degraded"
			break
		fi
	done

	log_must check_state $TESTPOOL $DADISK "FAULTED"

	# ZFSD can take up to 60 seconds to degrade an array in response to
	# errors (though it's usually faster).  
	for ((timeout=0; $timeout<10; timeout=$timeout+1)); do
		check_state $TESTPOOL "$sdev" "INUSE"
		spare_inuse=$?
		if [[ $spare_inuse == 0 ]]; then
			break
		fi
		$SLEEP 6
	done
	log_must $ZPOOL status $TESTPOOL
	log_must check_state $TESTPOOL "$sdev" "INUSE"

	# do cleanup
	destroy_pool $TESTPOOL
}


# Make sure that at DISK0 is using the da driver, and use
# that disk for inject errors
b=`basename $DISK0`
if test ${b%%[0-9]*} == da
then
	DADISK=$DISK0
	DADISKNUM=${b##da}
else
	log_unsupported "This test requires DISK0 to use the da driver"
fi

log_assert "A faulted vdev will be replaced by an available spare"

log_onexit cleanup

set_devs
typeset sdev=$DISK4
typeset pooldevs="$DISK0 $DISK1 $DISK2 $DISK3"

set -A my_keywords "mirror" "raidz1" "raidz2"
for keyword in "${my_keywords[@]}" ; do
	log_must create_pool $TESTPOOL $keyword $pooldevs spare $sdev
	log_must poolexists "$TESTPOOL"
	log_must $ZPOOL set autoreplace=on "$TESTPOOL"
	iterate_over_hotspares verify_assertion $sdev

	destroy_pool "$TESTPOOL"
done
